#include <stdio.h>
#include <string.h>
#include "common_porting.h"
#include "nrfx_spim.h"
#include "app_util_platform.h"
#include "nrf_gpio.h"
#include "nrf_delay.h"
#include "boards.h"
#include "app_error.h"
#include <string.h>
#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"
#include "bmi270_task.h"
#include "bmi270_wh.h"
#include "SEGGER_RTT.h"


struct bmi2_dev bmi270dev;

#if defined(ACC_GYRO)
struct bmi2_sensor_data sensor_data_AG[3] = { 0 };
#endif

void print_bmi270_rslt(int8_t rslt)
{
     switch (rslt)
      {
            case BMI2_OK:

                /* Do nothing */
                break;
            case BMI2_E_NULL_PTR:
                SEGGER_RTT_printf(0, "Error [%d] : Null pointer\r\n", rslt);
                break;
            case BMI2_E_COM_FAIL:
                SEGGER_RTT_printf(0, "Error [%d] : Communication failure\r\n", rslt);
                break;
            case BMI2_E_DEV_NOT_FOUND:
                SEGGER_RTT_printf(0, "Error [%d] : Device not found\r\n", rslt);
                break;
            case BMI2_E_INVALID_SENSOR:
                SEGGER_RTT_printf(0, "Error [%d] : Invalid sensor\r\n", rslt);
                break;
            case BMI2_E_SELF_TEST_FAIL:
                SEGGER_RTT_printf(0, "Warning [%d] : Self test failed\r\n", rslt);
                break;
            case BMI2_E_INVALID_INT_PIN:
                SEGGER_RTT_printf(0, "warning [%d] : invalid int pin\r\n", rslt);
                break;
            default:
                SEGGER_RTT_printf(0, "Error [%d] : Unknown error code\r\n", rslt);
                break;
      }
}

int8_t Open_BMI270_ACC(struct bmi2_dev *dev)
{
	int8_t rslt = BMI2_OK;
	uint8_t sensor_list = BMI2_ACCEL;
	/* Sensor configuration structure */
	struct bmi2_sens_config config = { 0 };

	/* Enable the selected sensors */
	rslt = bmi2_sensor_enable(&sensor_list, 1, dev);
	SEGGER_RTT_printf(0, "bmi2_sensor_enable BMI2_ACCEL\r\n");
	print_bmi270_rslt(rslt);

	config.type = sensor_list;
	
	/* Get the previous or default configuration settings */
	rslt = bmi2_get_sensor_config(&config, 1, dev);
	if (rslt == BMI2_OK) 
	{
		/* Update all or any of the accelerometer
		configurations */
		#if defined(SUPPORT_LOWPOWER)
		config.cfg.acc.filter_perf = BMI2_POWER_OPT_MODE;
		config.cfg.acc.bwp        = BMI2_ACC_OSR2_AVG2;
		#else
		config.cfg.acc.filter_perf = BMI2_PERF_OPT_MODE;
		config.cfg.acc.bwp        = BMI2_ACC_NORMAL_AVG4;
		#endif
		
		config.cfg.acc.odr      = BMI2_ACC_ODR_100HZ;
		config.cfg.acc.range     = BMI2_ACC_RANGE_8G;
		
		/* Set the configurations */
		rslt = bmi2_set_sensor_config(&config, 1, dev);
		if (rslt != BMI2_OK) 
		{
			SEGGER_RTT_printf(0, "accelerometer configuration failed, rslt=%d\r\n", rslt);
		} 
		else
		{
			SEGGER_RTT_printf(0, "Configuration set successfully\r\n");

			/* Get the configuration settings for validation */
			rslt = bmi2_get_sensor_config(&config, 1, dev);
			if (rslt == BMI2_OK) 
			{
				SEGGER_RTT_printf(0, "Get BMI2_ACCEL Configuration successful\r\n");
				SEGGER_RTT_printf(0, "Performance Mode = %d\r\n", config.cfg.acc.filter_perf);
				SEGGER_RTT_printf(0, "Bandwidth = %d\r\n", config.cfg.acc.bwp);
				SEGGER_RTT_printf(0, "ODR = %d\r\n", config.cfg.acc.odr);
				SEGGER_RTT_printf(0, "Range = %d\r\n", config.cfg.acc.range);
			}
		}
	}

	return rslt;
}

int8_t Close_BMI270_ACC(struct bmi2_dev *dev)
{
	int8_t rslt = BMI2_OK;
	uint8_t sensor_list = BMI2_ACCEL;

	/* Disable the selected sensors */
	rslt = bmi2_sensor_disable(&sensor_list, 1, dev);
	print_bmi270_rslt(rslt);

	return rslt;
}

int8_t Open_BMI270_GYRO(struct bmi2_dev *dev)
{
	int8_t rslt = BMI2_OK;
	uint8_t sensor_list = BMI2_GYRO;
	/* Sensor configuration structure */
	struct bmi2_sens_config config = { 0 };

	/* Enable the selected sensors */
	rslt = bmi2_sensor_enable(&sensor_list, 1, dev);
	print_bmi270_rslt(rslt);

	config.type = sensor_list;
	
	/* Get the previous or default configuration settings */
	rslt = bmi2_get_sensor_config(&config, 1, dev);
	if (rslt == BMI2_OK) 
	{
		config.cfg.gyr.odr = BMI2_GYR_ODR_100HZ;
		config.cfg.gyr.range = BMI2_GYR_RANGE_2000;
		config.cfg.gyr.ois_range = BMI2_GYR_OIS_2000;
		
		#if defined(SUPPORT_LOWPOWER)
		config.cfg.gyr.bwp = BMI2_GYR_OSR2_MODE;
		config.cfg.gyr.noise_perf = BMI2_POWER_OPT_MODE;
		#else
		config.cfg.gyr.bwp = BMI2_GYR_NORMAL_MODE;
		config.cfg.gyr.noise_perf = BMI2_PERF_OPT_MODE;
		#endif
	
		/* Set the configurations */
		rslt = bmi2_set_sensor_config(&config, 1, dev);
		if (rslt != BMI2_OK) 
		{
			SEGGER_RTT_printf(0, "accelerometer configuration failed\r\n");
		} 
		else
		{
			SEGGER_RTT_printf(0, "Configuration set successfully\r\n");

			/* Get the configuration settings for validation */
			rslt = bmi2_get_sensor_config(&config, 1, dev);
			if (rslt == BMI2_OK) 
			{
				SEGGER_RTT_printf(0, "Get BMI2_GYRO Configuration successful\r\n");
				SEGGER_RTT_printf(0, "Bandwidth = %d\r\n", config.cfg.gyr.bwp);
				SEGGER_RTT_printf(0, "ODR = %d\r\n", config.cfg.gyr.odr);
				SEGGER_RTT_printf(0, "Range = %d\r\n", config.cfg.gyr.range);
				SEGGER_RTT_printf(0, "OIS Range = %d\r\n", config.cfg.gyr.ois_range);
				SEGGER_RTT_printf(0, "Performance mode = %d\r\n", config.cfg.gyr.noise_perf);
			}
		}
	}

	return rslt;
}

int8_t Close_BMI270_GYRO(struct bmi2_dev *dev)
{
	int8_t rslt = BMI2_OK;
	uint8_t sensor_list = BMI2_GYRO;

	/* Disable the selected sensors */
	rslt = bmi2_sensor_disable(&sensor_list, 1, dev);
	print_bmi270_rslt(rslt);

	return rslt;
}

void BMI270_Print_ALLRegs(struct bmi2_dev *dev)
{
	uint8_t value;

	uint8_t reg_addr;

	for(reg_addr = 0; reg_addr < 0x7E; reg_addr++)
	{
		bmi2_get_regs(reg_addr, &value, 1, dev);
		SEGGER_RTT_printf(0, "0x%02X, value=0x%02X\r\n", reg_addr, value);
	}
}

int8_t Init_BMI270()
{
	int8_t rslt = BMI2_OK;
	uint8_t chipid;
	uint16_t ver_major;
	uint8_t ver_minor;

	struct bmi2_dev *dev;
	dev = &bmi270dev;

#if defined(USE_SPI_INTERFACE)
	#if defined(USE_DD30)
	dev->dev_id = 0x00,
	dev->intf = BMI2_SPI_INTERFACE,
	dev->read = (bmi2_read_fptr_t)DD30_SensorAPI_SPIx_Read,
	dev->write = (bmi2_write_fptr_t)DD30_SensorAPI_SPIx_Write,
	dev->delay_us = (bmi2_delay_fptr_t)DelayUs,
	dev->read_write_len = 32,
	dev->config_file_ptr = NULL;
	#elif defined(USE_BMI270_HEARABLE)
	#endif
#elif defined(USE_I2C_INTERFACE)
	#if defined(USE_DD30)
	dev->dev_id = BMI2_I2C_PRIM_ADDR,
	dev->intf = BMI2_I2C_INTERFACE,
	dev->read = (bmi2_read_fptr_t)DD30_SensorAPI_I2C_Read,
	dev->write = (bmi2_write_fptr_t)DD30_SensorAPI_I2C_Write,
	dev->delay_us = (bmi2_delay_fptr_t)DelayUs,
	dev->read_write_len = 32,
	dev->config_file_ptr = NULL;
	#elif defined(USE_BMI270_HEARABLE)
	#endif
#endif

	SEGGER_RTT_printf(0, "BMI270 initialize started\r\n");

	/* Variable to define soft reset value */
	uint8_t cmd = BMI2_SOFT_RESET_CMD;

	/* Reset bmi2 device */
	rslt = bmi2_set_regs(BMI2_CMD_REG_ADDR, &cmd, 1, dev);
	dev->delay_us(2000);

	bmi2_get_regs(0, &chipid, 1, dev);
	
	rslt = bmi270_wh_init(dev);
	if (rslt != BMI2_OK)
	{
		SEGGER_RTT_printf(0, "bmi270_wh_init() failed, error code: %d\r\n", rslt);
		return rslt;
	}
	else
	{
		/*
		rslt = bmi2_get_regs(BMI2_CHIP_ID_ADDR, &chipid, 1, dev);
		if (rslt != BMI2_OK) {
			SEGGER_RTT_printf(0, "read chip ID failed, error code: %d\r\n", rslt);
			return rslt;
		}*/

		SEGGER_RTT_printf(0, "bmi270_wh_init success, Chip ID:%02x\r\n", dev->chip_id);
	}

	SEGGER_RTT_printf(0, "BMI270 initialized successfully\r\n");

	rslt = bmi2_get_config_file_version(&ver_major, &ver_minor, dev);
	SEGGER_RTT_printf(0, "The firmware version: v%d.%d\r\n", ver_major, ver_minor);

	/* Disable advanced power save */
	rslt = bmi2_set_adv_power_save(BMI2_DISABLE, dev);

	Open_BMI270_ACC(dev);
	Open_BMI270_GYRO(dev);
	
	return rslt;
}

int8_t Read_ACC_RAWData(struct bmi2_sens_axes_data *accel)
{
	int8_t rslt;
	struct bmi2_dev *dev;
	dev = &bmi270dev;
	
	#if 1
	struct bmi2_sensor_data sensor_data;
	sensor_data.type = BMI2_ACCEL;   

	rslt = bmi2_get_sensor_data(&sensor_data, 1, dev);
	memcpy(accel, &sensor_data.sens_data.acc, sizeof(struct bmi2_sens_axes_data));
	#else
	rslt = read_accel_xyz(accel, dev);
	#endif

	return rslt;
}

int8_t Read_GYRO_RAWData(struct bmi2_sens_axes_data *gyro)
{
	int8_t rslt;
	struct bmi2_dev *dev;
	dev = &bmi270dev;

	#if 0
	struct bmi2_sensor_data sensor_data;
	sensor_data.type = BMI2_GYRO;   

	rslt = bmi2_get_sensor_data(&sensor_data, 1, dev);
	memcpy(gyro, &sensor_data.sens_data.gyr, sizeof(struct bmi2_sens_axes_data));
	#else
	rslt = read_gyro_xyz(gyro, dev);
	#endif

	return rslt;
}

int8_t Set_Acc_Range(int8_t range)
{
	int8_t rslt;
	struct bmi2_dev *dev;
	dev = &bmi270dev;

	if((range >= 0) && (range < 4))
	{
		rslt = bmi2_set_regs(0x41, &range, 1, dev);
	}
	else
	{
		return -1;
	}
	
	return rslt;
}

