#include <string.h>
#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdarg.h>
#include <math.h>
#include "sdk_config.h"
#include "app_util_platform.h"
#include "nrf_gpio.h"
#include "nrf_drv_spi.h"
#include "nrf_drv_twi.h"
#include "nrf_delay.h"
#include "boards.h"
#include "app_error.h"
#include <string.h>
#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"
#include "common_porting.h"
#include "SEGGER_RTT.h"


/* TWI instance ID. */

#define TWI_INSTANCE_ID     1
#define TWI_TIMEOUT                     10000 



/* TWI instance. */
static const nrf_drv_twi_t m_twi = NRF_DRV_TWI_INSTANCE(TWI_INSTANCE_ID);
volatile static bool s_twiTxDone = false;                   
volatile static bool s_twiRxDone = false;                 


static void mergeRegisterAndData(uint8_t *pTxBuf, uint8_t regAddr, uint8_t *pData, uint16_t dataLen)
{
    pTxBuf[0] = regAddr;
    memcpy((pTxBuf + 1), pData, dataLen);
}

/**
 * @brief TWI events handler.
 */
void twi_bst_handler(nrf_drv_twi_evt_t const * p_event, void * p_context)
{
	switch(p_event->type)
    {
        case NRF_DRV_TWI_EVT_DONE:
            switch(p_event->xfer_desc.type)
            {
                case NRF_DRV_TWI_XFER_TX:
                    s_twiTxDone = true;
                    break;
                case NRF_DRV_TWI_XFER_TXTX:
                    s_twiTxDone = true;
                    break;
                case NRF_DRV_TWI_XFER_RX:
                    s_twiRxDone = true;
                    break;
                case NRF_DRV_TWI_XFER_TXRX:
                    s_twiRxDone = true;
                    break;
                default:
                    break;
            }
            break;
        
        case NRF_DRV_TWI_EVT_ADDRESS_NACK:
            break;
        
        case NRF_DRV_TWI_EVT_DATA_NACK:
            break;
        
        default:
            break;
    }
}


#define SPI_INSTANCE  2 /**< SPI instance index. */
static const nrf_drv_spi_t spi = NRF_DRV_SPI_INSTANCE(SPI_INSTANCE);  /**< SPI instance. */
static volatile bool spi_xfer_done;  /**< Flag used to indicate that SPI instance completed the transfer. */

static uint8_t       m_tx_buf[32];           /**< TX buffer. */
static uint8_t       m_rx_buf[32];    /**< RX buffer. */
static uint8_t tx_length, rx_length;       /**< Transfer length. */

/**
 * @brief SPI user event handler.
 * @param event
 */
void spi_event_handler(nrf_drv_spi_evt_t const * p_event,
                       void *                    p_context)
{
    spi_xfer_done = true;
    NRF_LOG_INFO("Transfer completed.");
    if (m_rx_buf[0] != 0)
    {
        NRF_LOG_INFO(" Received:");
        NRF_LOG_HEXDUMP_INFO(m_rx_buf, strlen((const char *)m_rx_buf));
    }
}

					   
void DelayUs(uint32_t Delay)
{
   uint32_t i;

   while(Delay--)
   {
	   //for(i = 0; i < 84; i++)
	   for(i = 0; i < 16; i++)
	   {
		   ;
	   }
   }
}

void Open_Power(void)
{
#if defined(USE_DD30)
   nrf_gpio_cfg_output(DD30_VDD_SEL);
   nrf_gpio_pin_write(DD30_VDD_SEL, 0);
   nrf_gpio_cfg_output(DD30_VDD_PS_EN);
   nrf_gpio_pin_write(DD30_VDD_PS_EN, 1);
   nrf_gpio_cfg_output(DD30_VDDIO_PS_EN);
   nrf_gpio_pin_write(DD30_VDDIO_PS_EN, 1);
#elif defined(USE_BMI270_HEARABLE	)

#endif
   nrf_delay_ms(200);
}

void DD30_SPI_Enable(bool flag)
{
	if(flag == true)
	{
		nrf_gpio_pin_write(DD30_SPI_CSB, 0);
	}
	else
	{
		nrf_gpio_pin_write(DD30_SPI_CSB, 1);
	}
}

int8_t SPI_Init(void)
{
	nrf_drv_spi_config_t spi_config = NRF_DRV_SPI_DEFAULT_CONFIG;
#if defined(USE_DD30)	
	spi_config.ss_pin   = DD30_SPI_CSB;
    spi_config.miso_pin = DD30_SPI_MISO;
    spi_config.mosi_pin = DD30_SPI_MOSI;
    spi_config.sck_pin  = DD30_SPI_SCK;
	spi_config.irq_priority = APP_IRQ_PRIORITY_HIGH;
    APP_ERROR_CHECK(nrf_drv_spi_init(&spi, &spi_config, spi_event_handler, NULL));

	//nrf_gpio_cfg_output(DD30_SPI_CSB);
	//nrf_gpio_pin_write(DD30_SPI_CSB, 1);
#elif defined(USE_BMI270_HEARABLE	)

#endif	
	return 0;
}

int8_t I2C_Init(void)
{
	ret_code_t err_code;
	nrf_gpio_cfg_output(DD30_SPI_CSB);
	nrf_gpio_pin_write(DD30_SPI_CSB, 1);
	nrf_gpio_cfg_output(DD30_SPI_MISO);
	nrf_gpio_pin_write(DD30_SPI_MISO, 0);
#if defined(USE_DD30)	
	const nrf_drv_twi_config_t twi_bst_config = {
       .scl                = DD30_I2C_SCL,
       .sda                = DD30_I2C_SDA,
       .frequency          = NRF_DRV_TWI_FREQ_100K,
       .interrupt_priority = APP_IRQ_PRIORITY_HIGH,
       .clear_bus_init     = true
    };

    err_code = nrf_drv_twi_init(&m_twi, &twi_bst_config, twi_bst_handler, NULL);
    APP_ERROR_CHECK(err_code);
	nrf_drv_twi_enable(&m_twi);
	
	nrf_gpio_cfg_output(DD30_SPI_CSB);
	nrf_gpio_pin_write(DD30_SPI_CSB, 1);
	nrf_gpio_cfg_output(DD30_SPI_MISO);
	nrf_gpio_pin_write(DD30_SPI_MISO, 0);
	nrf_delay_ms(100);
#elif defined(USE_BMI270_HEARABLE	)

#endif	
	return 0;
}


int8_t DD30_SensorAPI_I2C_Write(uint8_t slaveAddr, uint8_t regAddr, uint8_t *pData, uint16_t dataLen)
{
    // This burst write function is not optimal and needs improvement.
    // The new SDK 11 TWI driver is not able to do two transmits without repeating the ADDRESS + Write bit byte
    uint32_t errCode;
    uint32_t timeout = TWI_TIMEOUT;
    // Merging MPU register address and p_data into one buffer.
    mergeRegisterAndData(m_tx_buf, regAddr, pData, dataLen);
    // Setting up transfer
    nrf_drv_twi_xfer_desc_t xferDesc;
    xferDesc.address = slaveAddr;
    xferDesc.type = NRF_DRV_TWI_XFER_TX;
    xferDesc.primary_length = dataLen + 1;
    xferDesc.p_primary_buf = m_tx_buf;

    // Transferring
    errCode = nrf_drv_twi_xfer(&m_twi, &xferDesc, 0);

    while((!s_twiTxDone) && --timeout);
    if(!timeout)
    {
        return -1;
    }
    s_twiTxDone = false;

    return 0; 
}

int8_t DD30_SensorAPI_I2C_Read(uint8_t slaveAddr, uint8_t regAddr, uint8_t *pData, uint16_t dataLen)
{
    uint32_t errCode;
    uint32_t timeout = TWI_TIMEOUT;

    errCode = nrf_drv_twi_tx(&m_twi, slaveAddr, &regAddr, 1, false);
    if(errCode != NRF_SUCCESS)
    {
        return -1;
    }

    while((!s_twiTxDone) && --timeout); 
    if(!timeout)
    {
        return NRF_ERROR_TIMEOUT;
    }
    s_twiTxDone = false;

    errCode = nrf_drv_twi_rx(&m_twi, slaveAddr, pData, dataLen);
    if(errCode != NRF_SUCCESS)
    {
        return -2;
    }

    timeout = TWI_TIMEOUT;
    while((!s_twiRxDone) && --timeout);
    if(!timeout)
    {
        return NRF_ERROR_TIMEOUT;
    }
    s_twiRxDone = false;
    
    return 0;
}


int8_t DD30_SensorAPI_SPIx_Read(uint8_t dev_addr, uint8_t reg_addr, uint8_t *reg_data, uint16_t count)
{
	nrfx_err_t error = 0;
	
	spi_xfer_done = false;

	//DD30_SPI_Enable(true);
	m_tx_buf[0] = reg_addr | 0x80;
	tx_length = 1;
	memset(m_rx_buf, 0, sizeof(m_rx_buf));
	rx_length = count +1;

	APP_ERROR_CHECK(nrf_drv_spi_transfer(&spi, m_tx_buf, tx_length, m_rx_buf, rx_length));

	while (!spi_xfer_done)
	{
		__WFE();
	}

	
	//DD30_SPI_Enable(false);
	memcpy(reg_data, &m_rx_buf[1], count); 

	return error;
}

int8_t DD30_SensorAPI_SPIx_Write(uint8_t dev_addr, uint8_t reg_addr, uint8_t *reg_data,uint16_t count)
{
	nrfx_err_t error = 0;

	spi_xfer_done = false;

	//DD30_SPI_Enable(true);
	memset(m_tx_buf, 0, sizeof(m_tx_buf));
	m_tx_buf[0] = reg_addr;
	memcpy(&m_tx_buf[1], reg_data, count);
	tx_length = count+1;
	rx_length = 0;

    APP_ERROR_CHECK(nrf_drv_spi_transfer(&spi, m_tx_buf, tx_length, m_rx_buf, rx_length));

    while (!spi_xfer_done)
    {
        __WFE();
    }
	//DD30_SPI_Enable(false);

	return error;
}



